@@ -13,12 +13,7 @@ import pandas as pd
 from matplotlib.offsetbox import AnnotationBbox, OffsetImage
 from matplotlib_scalebar.scalebar import ScaleBar
 from shapely import make_valid, reverse
-from shapely.geometry import (
-    LinearRing,
-    LineString,
-    MultiLineString,
-    Polygon,
-)
+from shapely.geometry import LinearRing, LineString, MultiLineString, Point, Polygon
 from shapely.ops import linemerge
 
 from .errors import CrsError, InputError, SlopeError
@@ -26,7 +21,10 @@ from .utils.clipping import clip_rays_by_polygon, remove_intersecting_lines
 from .utils.perpendicular import calculate_slopes
 from .utils.resolution import redistribute_vertices
 from .utils.shoal_break import line_of_breaking, line_of_shoaling
-from .utils.smoother import cast_smoothed_rays, determine_ray_direction
+from .utils.smoother import (
+    cast_snell_rays,
+    determine_ray_direction,
+)
 
 
 class ShorelineAnalyzer:
@@ -63,7 +61,9 @@ class ShorelineAnalyzer:
         wave_height,
         ray_resolution=50.0,
         ray_length=10000,
-        smoothing_window=250.0,
+        smoothing_window=250.0,  # no longer needed, keeping for compat
+        origin_angle=0.0,
+        origin_distance=1000,
     ):
         """
         Initialize the ShorelineAnalyzer.
@@ -113,6 +113,8 @@ class ShorelineAnalyzer:
             err = "The smoothing window should be positive. Try 250.0, unless you're sure you know what you're doing"
             raise InputError(err)
         self.smoothing_window = smoothing_window
+        self.origin_angle = origin_angle
+        self.origin_distance = origin_distance
         self.log = {
             "timestamp": datetime.datetime.now(datetime.timezone.utc)
             .replace(microsecond=0)
@@ -138,24 +140,35 @@ class ShorelineAnalyzer:
         return isobath
 
     def _cast_rays(self):
+        # If direction is "right", rays should go right to left (right=False)
+        # If direction is "left", rays should go left to right (right=True)
         if self.direction == "right":
-            right = True
-        else:
             right = False
-        rays = cast_smoothed_rays(
+        else:
+            right = True
+        all_rays = cast_snell_rays(
             self.tideline,
-            self.smoothing_window,
-            self.ray_length,
+            ray_length=self.ray_length,
+            origin_distance=self.origin_distance,
+            origin_angle=self.origin_angle,
             right=right,
         )
+        rays = all_rays[0]
+        origins = all_rays[1]
         self.rays = gpd.GeoDataFrame(geometry=[ray for ray in rays], crs=self.crs)
+        self.origin_rays = gpd.GeoDataFrame(
+            geometry=[ray for ray in origins], crs=self.crs
+        )
 
     def _trim(self):
         self.trimmed = clip_rays_by_polygon(self.rays, self.shoreline)
         self.trimmed["length"] = self.trimmed.geometry.length
 
     def _remove_intersections(self):
-        self.trimmed = remove_intersecting_lines(self.trimmed)
+        self.intersections = []
+        res = remove_intersecting_lines(self.trimmed)
+        self.trimmed = res[0]
+        self.intersections = res[1]
 
     def _assign_depths(self):
         self.trimmed["start_depth"] = self.lat
@@ -167,6 +180,9 @@ class ShorelineAnalyzer:
         ) / self.trimmed["length"]
         self.trimmed["slope_degrees"] = calculate_slopes(self.trimmed)
         self.trimmed["slope_radians"] = np.deg2rad(self.trimmed["slope_degrees"])
+        self.trimmed["slope_degrees_normalised"] = (
+            self.trimmed["slope_degrees"] - np.min(self.trimmed["slope_degrees"])
+        ) / np.ptp(self.trimmed["slope_degrees"])
 
     def _shoaling(self):
         self.shoal_line = line_of_shoaling(self.wave_period, self.trimmed, start=False)
@@ -174,6 +190,112 @@ class ShorelineAnalyzer:
     def _breaking(self):
         self.break_line = line_of_breaking(self.wave_height, self.trimmed)
 
+    def _calculate_point_distances(self):
+        """
+        Calculate distances from each point in self.trimmed to equivalent points
+        on breaking and shoaling lines, and the width between them.
+        Also calculate normalized values and geometric mean.
+        """
+        # Extract coordinates from the shoaling and breaking LineStrings
+        shoal_coords = list(self.shoal_line.geometry.iloc[0].coords)
+        break_coords = list(self.break_line.geometry.iloc[0].coords)
+
+        # Create GeoDataFrames with Point geometries for vectorized operations
+        shoal_points = gpd.GeoDataFrame(
+            geometry=[Point(coord) for coord in shoal_coords],
+            crs=self.crs,
+            index=self.trimmed.index,
+        )
+
+        break_points = gpd.GeoDataFrame(
+            geometry=[Point(coord) for coord in break_coords],
+            crs=self.crs,
+            index=self.trimmed.index,
+        )
+
+        # Get endpoints of trimmed rays (at HAT) as a GeoSeries
+        trimmed_endpoints = self.trimmed.geometry.apply(
+            lambda geom: Point(geom.coords[-1])
+        )
+
+        # Calculate distances using vectorized operations
+        self.trimmed["distance_to_breaking"] = trimmed_endpoints.distance(
+            break_points.geometry
+        )
+        self.trimmed["distance_to_shoaling"] = trimmed_endpoints.distance(
+            shoal_points.geometry
+        )
+        self.trimmed["shoal_break_width"] = shoal_points.geometry.distance(
+            break_points.geometry
+        )
+
+        # Calculate normalized values (0-1 range using min-max normalization)
+        # Invert the normalization - smaller distances = higher scores for impact intensity
+        self.trimmed["distance_to_breaking_normalised"] = 1 - (
+            (
+                self.trimmed["distance_to_breaking"]
+                - self.trimmed["distance_to_breaking"].min()
+            )
+            / (
+                self.trimmed["distance_to_breaking"].max()
+                - self.trimmed["distance_to_breaking"].min()
+            )
+        )
+
+        self.trimmed["distance_to_shoaling_normalised"] = 1 - (
+            (
+                self.trimmed["distance_to_shoaling"]
+                - self.trimmed["distance_to_shoaling"].min()
+            )
+            / (
+                self.trimmed["distance_to_shoaling"].max()
+                - self.trimmed["distance_to_shoaling"].min()
+            )
+        )
+
+        self.trimmed["shoal_break_width_normalised"] = 1 - (
+            (
+                self.trimmed["shoal_break_width"]
+                - self.trimmed["shoal_break_width"].min()
+            )
+            / (
+                self.trimmed["shoal_break_width"].max()
+                - self.trimmed["shoal_break_width"].min()
+            )
+        )
+
+        # Calculate geometric mean of the four normalized columns
+        # Geometric mean = (a * b * c * d)^(1/4)
+        self.trimmed["geom_mean_normalised"] = np.power(
+            self.trimmed["distance_to_breaking_normalised"]
+            * self.trimmed["distance_to_shoaling_normalised"]
+            * self.trimmed["shoal_break_width_normalised"]
+            * self.trimmed["slope_degrees_normalised"],
+            0.25,
+        )
+
+    def _build_score_line(self):
+        """
+        Build a LineString where each vertex corresponds to a trimmed ray endpoint
+        with the geometric mean score as a property.
+        """
+        # Extract endpoints of trimmed rays
+        points = []
+        for _, row in self.trimmed.iterrows():
+            # Get the endpoint (at HAT)
+            endpoint = row.geometry.coords[-1]
+            points.append(endpoint)
+
+        # Create LineString from the points
+        score_line = LineString(points)
+
+        # Create GeoDataFrame with the score property
+        self.score_line = gpd.GeoDataFrame(
+            geometry=[score_line],
+            crs=self.crs,
+            data={"score": [self.trimmed["geom_mean_normalised"].values]},
+        )
+
     def _build_intertidal(self):
         end = LineString(
             [
@@ -218,10 +340,13 @@ class ShorelineAnalyzer:
         """
         self._cast_rays()
         self._trim()
+        self._remove_intersections()
         self._assign_depths()
         self._assign_slopes()
         self._shoaling()
         self._breaking()
+        self._calculate_point_distances()
+        self._build_score_line()
         self._build_intertidal()
         result = AnalysisResult(
             self.crs,
@@ -235,9 +360,13 @@ class ShorelineAnalyzer:
             gpd.GeoDataFrame(geometry=[self.tideline], crs=self.crs),
             self.shoreline,
             self.trimmed,
+            self.intersections,
             self.shoal_line,
             self.break_line,
             self.intertidal,
+            self.smoothing_window,
+            self.origin_rays,
+            self.score_line,
         )
         return result
 
@@ -258,11 +387,16 @@ class AnalysisResult:
         lat_line,
         shore_line,
         slopes,
+        intersections,
         shoaling,
         breaking,
         intertidal,
+        window,
+        origins,
+        score,
     ):
         super().__init__()
+        self.intersections = intersections
         self.metadata = dict()
         self.metadata["timestamp"] = (
             datetime.datetime.now(datetime.timezone.utc)
@@ -277,14 +411,34 @@ class AnalysisResult:
         self.metadata["wave_period"] = period
         self.metadata["wave_height"] = height
         self.metadata["ray_resolution"] = res
+        self.metadata["smoothing_window"] = window
         self.lat_line = lat_line
         self.shore_line = shore_line
+        self.origin_rays = origins
         self.slopes = slopes
         self.shoaling = shoaling
         self.breaking = breaking
         self.intertidal = intertidal
+        self.score = score
         self.pcbreaks = [10, 25, 50, 75, 90, 95]
+        self.metric_description = [
+            "Mean Slope",
+            "Median Slope",
+            "Std Deviation",
+            "Mean Angle (°)",
+            "Median Angle (°)",
+            "Basic Intertidal Gradient",
+            "Basic Intertidal Angle (°)",
+            "Weighted Intertidal Gradient",
+            "Weighted Intertidal Angle (°)",
+            "Mean Segment Gradient",
+            "Mean Segment Angle (°)",
+            "Area Weighted Gradient",
+            "Area Weighted Angle (°)",
+            "Slope Variation Coefficient",
+        ]
         self.metrics = self._intertidal_gradient_metrics()
+        self.friendly_metrics = self._build_metrics_dict()
 
     def _intertidal_gradient_metrics(self):
         """
@@ -454,6 +608,28 @@ class AnalysisResult:
 
         return metrics
 
+    def _build_metrics_dict(self):
+        """Friendly subset of the full intertidal gradient metric set"""
+        return {
+            "Metric": self.metric_description,
+            "Value": [
+                self.metrics["mean_slope"],
+                self.metrics["median_slope"],
+                self.metrics["std_slope"],
+                self.metrics["mean_angle"],
+                self.metrics["median_angle"],
+                self.metrics["basic_intertidal_gradient"],
+                self.metrics["basic_intertidal_angle"],
+                self.metrics["weighted_intertidal_gradient"],
+                self.metrics["weighted_intertidal_angle"],
+                self.metrics["mean_segment_gradient"],
+                self.metrics["mean_segment_angle"],
+                self.metrics["area_weighted_gradient"],
+                self.metrics["area_weighted_angle"],
+                self.metrics["slope_variation_coef"],
+            ],
+        }
+
     def visualize_coastal_slopes(self, plot_indices=True):
         """
         Visualize analyzed coastal slopes with multiple plots.
@@ -487,6 +663,7 @@ class AnalysisResult:
         wd = self.slopes.plot(  # noqa: F841
             ax=ax1,
             column="slope_degrees",
+            cmap="viridis",
             legend=True,
             legend_kwds={
                 "label": "Slope (degrees)",
@@ -494,10 +671,12 @@ class AnalysisResult:
             zorder=0,
         )
 
-        min_x, max_x, min_y, max_y = self._get_expanded_bounds(500)
+        min_x, max_x, min_y, max_y = self._get_expanded_bounds(4000)
         ax1.set_xlim(min_x, max_x)
         ax1.set_ylim(min_y, max_y)
 
+        self.origin_rays.plot(ax=ax1, color="green", lw=0.5, label="Wave origin")
+
         latp = self.lat_line.plot(  # noqa: F841
             ax=ax1, linestyle="--", color="#39FF14", lw=1.0, label="LAT"
         )
@@ -514,24 +693,68 @@ class AnalysisResult:
             zorder=8,
         )
         breaking = self.breaking.plot(  # noqa: F841
-            ax=ax1, color="red", lw=0.5, label="Breaking", zorder=9
+            ax=ax1, color="red", lw=0.5, zorder=9, label="Breaking"
         )
 
+        # slope_endpoints = gpd.GeoDataFrame(
+        #     geometry=pd.concat(
+        #         [self.slopes.geometry.apply(lambda geom: Point(geom.coords[-1]))]
+        #     ),
+        #     crs=self.slopes.crs,
+        # )
+        # # slope_endpoints.plot(
+        # #     ax=ax1,
+        # #     color="blue",
+        # #     zorder=0,
+        # # )
+
+        if len(self.intersections):
+            intersection_endpoints = gpd.GeoDataFrame(
+                geometry=pd.concat(
+                    [
+                        self.intersections.geometry.apply(
+                            lambda geom: Point(geom.coords[-1])
+                        )
+                    ]
+                ),
+                crs=self.intersections.crs,
+            )
+
+            self.intersections.plot(
+                ax=ax1,
+                # color="#033500",
+                color="#8f00ff",
+                lw=0.1,
+                zorder=9,
+                label="Ray intersections",
+                legend=True,
+                legend_kwds={
+                    "Intersecting Lines",
+                },
+            )
+
+            # Extract endpoints from intersection LineString geometries
+            intersection_endpoints.plot(
+                ax=ax1,
+                color="red",
+                markersize=2,
+                zorder=10,
+                label="Intersection endpoints",
+            )
+
         ax1.set_title(
-            f"HAT {self.metadata['hat']}, LAT {self.metadata['lat']} , Period: {self.metadata['wave_period']} s, Wave height: {self.metadata['wave_height']} m"
+            f"Inputs (metres, seconds):\nSmoothing Window: {self.metadata['smoothing_window']}, HAT {self.metadata['hat']}, LAT {self.metadata['lat']} , Period: {self.metadata['wave_period']}, Wave height: {self.metadata['wave_height']}",
+            fontsize=8,
         )
         ax1.axis("off")
         fig1.patch.set_alpha(1)
         fig1.set_dpi(200)
         ax1.patch.set_alpha(1)
-        leg = ax1.legend(loc="best")
+        leg = ax1.legend(loc="best", fontsize=8)
         leg.set_zorder(10)
         for line in leg.get_lines():
             line.set_linewidth(2.0)
 
-        ax1.set_title(
-            f"Coastal Profile with Sampling Rays (Ray resolution: {self.metadata['ray_resolution']} m)"
-        )
         ax1.set_aspect("equal")
 
         # create logo imagebox
@@ -677,44 +900,7 @@ class AnalysisResult:
         """
         Return summary statistics for the analysed shoreline
         """
-        print(
-            pd.DataFrame(
-                {
-                    "Metric": [
-                        "Mean Slope",
-                        "Median Slope",
-                        "Std Deviation",
-                        "Mean Angle (°)",
-                        "Median Angle (°)",
-                        "Basic Intertidal Gradient (Short and Wright)",
-                        "Basic Intertidal Angle (°)",
-                        "Weighted Intertidal Gradient (Plant et al)",
-                        "Weighted Intertidal Angle (°)",
-                        "Mean Segment Gradient (Masselink & Hegge)",
-                        "Mean Segment Angle (°)",
-                        "Area Weighted Gradient (Hegge et al)",
-                        "Area Weighted Angle (°)",
-                        "Slope Variation Coefficient",
-                    ],
-                    "Value": [
-                        self.metrics["mean_slope"],
-                        self.metrics["median_slope"],
-                        self.metrics["std_slope"],
-                        self.metrics["mean_angle"],
-                        self.metrics["median_angle"],
-                        self.metrics["basic_intertidal_gradient"],
-                        self.metrics["basic_intertidal_angle"],
-                        self.metrics["weighted_intertidal_gradient"],
-                        self.metrics["weighted_intertidal_angle"],
-                        self.metrics["mean_segment_gradient"],
-                        self.metrics["mean_segment_angle"],
-                        self.metrics["area_weighted_gradient"],
-                        self.metrics["area_weighted_angle"],
-                        self.metrics["slope_variation_coef"],
-                    ],
-                }
-            ).to_string(index=False)
-        )
+        print(pd.DataFrame(self._build_metrics_dict()).to_string(index=False))
         # Print percentiles separately
         print("\n\t\t\t\tSlope Angle Percentiles (°):")
         for p, v in zip(self.pcbreaks, self.metrics["percentiles"]):
